MODULE eosbn2
   !!==============================================================================
   !!                       ***  MODULE  eosbn2  ***
   !! Ocean diagnostic variable : equation of state - in situ and potential density
   !!                                               - Brunt-Vaisala frequency
   !!==============================================================================
   !! History :  OPA  ! 1989-03  (O. Marti)  Original code
   !!            6.0  ! 1994-07  (G. Madec, M. Imbard)  add bn2
   !!            6.0  ! 1994-08  (G. Madec)  Add Jackett & McDougall eos
   !!            7.0  ! 1996-01  (G. Madec)  statement function for e3
   !!            8.1  ! 1997-07  (G. Madec)  density instead of volumic mass
   !!             -   ! 1999-02  (G. Madec, N. Grima) semi-implicit pressure gradient
   !!            8.2  ! 2001-09  (M. Ben Jelloul)  bugfix on linear eos
   !!   NEMO     1.0  ! 2002-10  (G. Madec)  add eos_init
   !!             -   ! 2002-11  (G. Madec, A. Bozec)  partial step, eos_insitu_2d
   !!             -   ! 2003-08  (G. Madec)  F90, free form
   !!            3.0  ! 2006-08  (G. Madec)  add tfreez function (now eos_fzp function)
   !!            3.3  ! 2010-05  (C. Ethe, G. Madec)  merge TRC-TRA
   !!             -   ! 2010-10  (G. Nurser, G. Madec)  add alpha/beta used in ldfslp
   !!            3.7  ! 2012-03  (F. Roquet, G. Madec)  add primitive of alpha and beta used in PE computation
   !!             -   ! 2012-05  (F. Roquet)  add Vallis and original JM95 equation of state
   !!             -   ! 2013-04  (F. Roquet, G. Madec)  add eos_rab, change bn2 computation and reorganize the module
   !!             -   ! 2014-02  (F. Roquet)  add TEOS-10, S-EOS, and modify EOS-80
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   eos            : generic interface of the equation of state
   !!   eos_insitu     : Compute the in situ density
   !!   eos_insitu_pot : Compute the insitu and surface referenced potential volumic mass
   !!   eos_insitu_2d  : Compute the in situ density for 2d fields
   !!   bn2            : Compute the Brunt-Vaisala frequency
   !!   eos_rab        : generic interface of in situ thermal/haline expansion ratio 
   !!   eos_rab_3d     : compute in situ thermal/haline expansion ratio
   !!   eos_rab_2d     : compute in situ thermal/haline expansion ratio for 2d fields
   !!   eos_fzp        : freezing temperature
   !!   eos_init       : set eos parameters (namelist)
   !!----------------------------------------------------------------------
   USE dom_oce         ! ocean space and time domain
   USE phycst          ! physical constants
   !
   USE in_out_manager  ! I/O manager
   USE lib_mpp         ! MPP library
   USE lib_fortran     ! Fortran utilities (allows no signed zero when 'key_nosignedzero' defined)  
   USE prtctl          ! Print control
   USE wrk_nemo        ! Memory Allocation
   USE timing          ! Timing

   IMPLICIT NONE
   PRIVATE

   !                   !! * Interface
   INTERFACE eos
      MODULE PROCEDURE eos_insitu, eos_insitu_pot, eos_insitu_2d
   END INTERFACE
   !
   INTERFACE eos_rab
      MODULE PROCEDURE rab_3d, rab_2d
   END INTERFACE
   !
   PUBLIC   eos            ! called by step, istate, tranpc and zpsgrd modules
   PUBLIC   bn2            ! called by step module
   PUBLIC   eos_rab        ! called by ldfslp, zdfddm, trabbl
   PUBLIC   eos_pt_from_ct ! called by sbcssm
   PUBLIC   eos_fzp        ! called by traadv_cen2 and sbcice_... modules
   PUBLIC   eos_pen        ! used for pe diagnostics in trdpen module
   PUBLIC   eos_init       ! called by istate module

   !                                          !!* Namelist (nameos) *
   INTEGER , PUBLIC ::   nn_eos   = 0         !: = 0/1/2 type of eq. of state and Brunt-Vaisala frequ.
   LOGICAL , PUBLIC ::   ln_useCT  = .FALSE.  ! determine if eos_pt_from_ct is used to compute sst_m

   !                                   !!!  simplified eos coefficients
   ! default value: Vallis 2006
   REAL(wp) ::   rn_a0      = 1.6550e-1_wp     ! thermal expansion coeff. 
   REAL(wp) ::   rn_b0      = 7.6554e-1_wp     ! saline  expansion coeff. 
   REAL(wp) ::   rn_lambda1 = 5.9520e-2_wp     ! cabbeling coeff. in T^2        
   REAL(wp) ::   rn_lambda2 = 5.4914e-4_wp     ! cabbeling coeff. in S^2        
   REAL(wp) ::   rn_mu1     = 1.4970e-4_wp     ! thermobaric coeff. in T  
   REAL(wp) ::   rn_mu2     = 1.1090e-5_wp     ! thermobaric coeff. in S  
   REAL(wp) ::   rn_nu      = 2.4341e-3_wp     ! cabbeling coeff. in theta*salt  
   
   ! EOS parameters
   REAL(wp) ::   EOS111 , EOS211 , EOS311 , EOS411 , EOS511 , EOS611 , EOS711
   REAL(wp) ::   EOS121 , EOS221 , EOS321 , EOS421 , EOS521 , EOS621
   REAL(wp) ::   EOS131 , EOS231 , EOS331 , EOS431 , EOS531
   REAL(wp) ::   EOS141 , EOS241 , EOS341 , EOS441
   REAL(wp) ::   EOS151 , EOS251 , EOS351
   REAL(wp) ::   EOS161 , EOS261
   REAL(wp) ::   EOS171
   REAL(wp) ::   EOS112 , EOS212 , EOS312 , EOS412 , EOS512
   REAL(wp) ::   EOS122 , EOS222 , EOS322 , EOS422
   REAL(wp) ::   EOS132 , EOS232 , EOS332
   REAL(wp) ::   EOS142 , EOS242
   REAL(wp) ::   EOS152
   REAL(wp) ::   EOS113 , EOS213 , EOS313
   REAL(wp) ::   EOS123 , EOS223
   REAL(wp) ::   EOS133
   
   ! ALPHA parameters
   REAL(wp) ::   ALP111 , ALP211 , ALP311 , ALP411 , ALP511 , ALP611
   REAL(wp) ::   ALP121 , ALP221 , ALP321 , ALP421 , ALP521
   REAL(wp) ::   ALP131 , ALP231 , ALP331 , ALP431
   REAL(wp) ::   ALP141 , ALP241 , ALP341
   REAL(wp) ::   ALP151 , ALP251
   REAL(wp) ::   ALP161
   REAL(wp) ::   ALP112 , ALP212 , ALP312 , ALP412
   REAL(wp) ::   ALP122 , ALP222 , ALP322
   REAL(wp) ::   ALP132 , ALP232
   REAL(wp) ::   ALP142
   REAL(wp) ::   ALP113 , ALP213
   REAL(wp) ::   ALP123
   
   ! BETA parameters
   REAL(wp) ::   BET111 , BET211 , BET311 , BET411 , BET511 , BET611
   REAL(wp) ::   BET121 , BET221 , BET321 , BET421 , BET521
   REAL(wp) ::   BET131 , BET231 , BET331 , BET431
   REAL(wp) ::   BET141 , BET241 , BET341
   REAL(wp) ::   BET151 , BET251
   REAL(wp) ::   BET161
   REAL(wp) ::   BET112 , BET212 , BET312 , BET412
   REAL(wp) ::   BET122 , BET222 , BET322
   REAL(wp) ::   BET132 , BET232
   REAL(wp) ::   BET142
   REAL(wp) ::   BET113 , BET213
   REAL(wp) ::   BET123

   ! PEN parameters
   REAL(wp) ::   PEN112 , PEN212 , PEN312 , PEN412 , PEN512
   REAL(wp) ::   PEN122 , PEN222 , PEN322 , PEN422
   REAL(wp) ::   PEN132 , PEN232 , PEN332
   REAL(wp) ::   PEN142 , PEN242
   REAL(wp) ::   PEN152
   REAL(wp) ::   PEN113 , PEN213 , PEN313
   REAL(wp) ::   PEN123 , PEN223
   REAL(wp) ::   PEN133
   
   ! ALPHA_PEN parameters
   REAL(wp) ::   APE112 , APE212 , APE312 , APE412
   REAL(wp) ::   APE122 , APE222 , APE322
   REAL(wp) ::   APE132 , APE232
   REAL(wp) ::   APE142
   REAL(wp) ::   APE113 , APE213
   REAL(wp) ::   APE123

   ! BETA_PEN parameters
   REAL(wp) ::   BPE112 , BPE212 , BPE312 , BPE412
   REAL(wp) ::   BPE122 , BPE222 , BPE322
   REAL(wp) ::   BPE132 , BPE232
   REAL(wp) ::   BPE142
   REAL(wp) ::   BPE113 , BPE213
   REAL(wp) ::   BPE123

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.7 , NEMO Consortium (2014)
   !! $Id$
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE eos_insitu( pts, prd, pdep )
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE eos_insitu  ***
      !!
      !! ** Purpose :   Compute the in situ density (ratio rho/rau0) from
      !!       potential temperature and salinity using an equation of state
      !!       defined through the namelist parameter nn_eos.
      !!
      !! ** Method  :   prd(t,s,z) = ( rho(t,s,z) - rau0 ) / rau0
      !!         with   prd    in situ density anomalie     no units
      !!                t      potential temperature        Celsius
      !!                s      salinity                     psu
      !!                z      depth                        meters
      !!                rho    in situ density              kg/m^3
      !!                rau0   reference density            kg/m^3
      !!
      !!     nn_eos = -1 : polynomial TEOS-10 equation of state is used for rho(t,s,z).
      !!         Check value: rho = 1028.21583 kg/m^3 for p=3000 dbar, t=3 Celcius, s=35.5 psu
      !!
      !!     nn_eos =  0 : Jackett and McDougall (1995) equation of state is used for rho(t,s,z)
      !!            where the density anomaly of a water parcel at t=4, s=35 moved adiabatically to surface
      !!            has been removed: rho = rho_JM95 - ( rho_JM95(4,35,z) - rho_JM95(4,35,0) )
      !!         Check value: rho = 1028.34976 kg/m^3 for p=3000 dbar, t=3 Celcius, s=35.5 psu
      !!         Check value: rho_JM95 = 1041.83267 kg/m^3 for p=3000 dbar, t=3 Celcius, s=35.5 psu
      !!
      !!     nn_eos =  1 : simplified equation of state
      !!              prd(t,s,z) = ( -a0*(1+lambda/2*(T-T0)+mu*z+nu*(S-S0))*(T-T0) + b0*(S-S0) ) / rau0
      !!              linear case function of T only: rn_alpha<>0, other coefficients = 0
      !!              linear eos function of T and S: rn_alpha and rn_beta<>0, other coefficients=0
      !!              Vallis like equation: use default values of coefficients
      !!
      !! ** Action  :   compute prd , the in situ density (no units)
      !!
      !! References :   Jackett and McDougall, J. Atmos. Ocean. Tech., 1995
      !!                Vallis, Atmospheric and Oceanic Fluid Dynamics, 2006
      !!                TEOS-10 Manual, 2010
      !!                Roquet, Ocean Modelling, in preparation (2013)
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(in   ) ::   pts   ! 1 : potential temperature  [Celcius]
      !                                                               ! 2 : salinity               [psu]
      REAL(wp), DIMENSION(jpi,jpj,jpk     ), INTENT(  out) ::   prd   ! in situ density            [-]
      REAL(wp), DIMENSION(jpi,jpj,jpk     ), INTENT(in   ) ::   pdep  ! depth                      [m]
      !
      INTEGER  ::   ji, jj, jk                ! dummy loop indices
      REAL(wp) ::   zt , zh , zs , zsr, ztm   ! local scalars
      REAL(wp) ::   zn , zn0, zn1, zn2        !   -      -
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )   CALL timing_start('eos')
      !
      SELECT CASE( nn_eos )
      !
      CASE( -1, 0 )                !==  polynomial TEOS-10 / EOS-80 ==!
         !
         DO jk = 1, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  !
                  zh  = pdep(ji,jj,jk)                                  ! depth
                  zt  = pts (ji,jj,jk,jp_tem)                           ! temperature
                  zsr = SQRT( MAX( pts(ji,jj,jk,jp_sal) , 0.1_wp ) )    ! square root salinity
                  ztm = tmask(ji,jj,jk)                                 ! tmask
                  !
                  zn2  = (   EOS133*zsr  + EOS223*zt + EOS123 )*zsr + ( EOS313*zt + EOS213 )*zt + EOS113
                  !
                  zn1  = ( ( ( EOS152*zsr   &
                     &   + EOS242*zt + EOS142 )*zsr   &
                     &   + ( EOS332*zt + EOS232 )*zt + EOS132 )*zsr   &
                     &   + ( ( EOS422*zt + EOS322 )*zt + EOS222 )*zt + EOS122 )*zsr   &
                     &   + ( ( ( EOS512*zt + EOS412 )*zt + EOS312 )*zt + EOS212 )*zt + EOS112
                  !
                  zn0  = ( ( ( ( ( EOS171*zsr   &
                     &   + EOS261*zt + EOS161 )*zsr   &
                     &   + ( EOS351*zt + EOS251 )*zt + EOS151 )*zsr   &
                     &   + ( ( EOS441*zt + EOS341 )*zt + EOS241 )*zt + EOS141 )*zsr   &
                     &   + ( ( ( EOS531*zt + EOS431 )*zt + EOS331 )*zt + EOS231 )*zt + EOS131 )*zsr   &
                     &   + ( ( ( ( EOS621*zt + EOS521 )*zt + EOS421 )*zt + EOS321 )*zt + EOS221 )*zt + EOS121 )*zsr   &
                     &   + ( ( ( ( ( EOS711*zt + EOS611 )*zt + EOS511 )*zt + EOS411 )*zt + EOS311 )*zt + EOS211 )*zt + EOS111
                  !
                  zn   = ( zn2 * zh + zn1 ) * zh + zn0
                  !                                               
                  prd(ji,jj,jk) = (  zn * r1_rau0 - 1._wp  ) * ztm  ! density anomaly (masked)
               END DO
            END DO
         END DO
         !
      CASE( 1 )                !==  simplified EOS  ==!
         !
         DO jk = 1, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zt  = pts  (ji,jj,jk,jp_tem) - 10._wp
                  zs  = pts  (ji,jj,jk,jp_sal) - 35._wp
                  zh  = pdep (ji,jj,jk)
                  ztm = tmask(ji,jj,jk)
                  !
                  zn =  - rn_a0 * ( 1._wp + 0.5_wp*rn_lambda1*zt + rn_mu1*zh ) * zt   &
                     &  + rn_b0 * ( 1._wp - 0.5_wp*rn_lambda2*zs - rn_mu2*zh ) * zs   &
                     &  - rn_nu * zt * zs
                  !                                 
                  prd(ji,jj,jk) = zn * r1_rau0 * ztm                ! density anomaly (masked)
               END DO
            END DO
         END DO
         !
      END SELECT
      !
      IF(ln_ctl)   CALL prt_ctl( tab3d_1=prd, clinfo1=' eos  : ', ovlap=1, kdim=jpk )
      !
      IF( nn_timing == 1 )   CALL timing_stop('eos')
      !
   END SUBROUTINE eos_insitu


   SUBROUTINE eos_insitu_pot( pts, prd, prhop, pdep )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE eos_insitu_pot  ***
      !!
      !! ** Purpose :   Compute the in situ density (ratio rho/rau0) and the
      !!      potential volumic mass (Kg/m3) from potential temperature and
      !!      salinity fields using an equation of state defined through the
      !!     namelist parameter nn_eos.
      !!
      !! ** Action  : - prd  , the in situ density (no units)
      !!              - prhop, the potential volumic mass (Kg/m3)
      !!
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(in   ) ::   pts    ! 1 : potential temperature  [Celcius]
      !                                                                ! 2 : salinity               [psu]
      REAL(wp), DIMENSION(jpi,jpj,jpk     ), INTENT(  out) ::   prd    ! in situ density            [-]
      REAL(wp), DIMENSION(jpi,jpj,jpk     ), INTENT(  out) ::   prhop  ! potential density (surface referenced)
      REAL(wp), DIMENSION(jpi,jpj,jpk     ), INTENT(in   ) ::   pdep   ! depth                      [m]
      !
      INTEGER  ::   ji, jj, jk                ! dummy loop indices
      REAL(wp) ::   zt , zh , zs , zsr, ztm   ! local scalars
      REAL(wp) ::   zn , zn0, zn1, zn2        !   -      -
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )   CALL timing_start('eos-p')
      !
      SELECT CASE ( nn_eos )
      !
      CASE( -1, 0 )                !==  polynomial TEOS-10 / EOS-80 ==!
         !
         DO jk = 1, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  !
                  zh  = pdep (ji,jj,jk)                                 ! depth
                  zt  = pts  (ji,jj,jk,jp_tem)                          ! conservative temperature
                  zsr = SQRT( MAX( pts(ji,jj,jk,jp_sal) , 0.1_wp ) )    ! square root salinity
                  ztm = tmask(ji,jj,jk)                                 ! tmask
                  !
                  zn2  = (   EOS133*zsr  + EOS223*zt + EOS123 )*zsr + ( EOS313*zt + EOS213 )*zt + EOS113
                  !
                  zn1  = ( ( ( EOS152*zsr   &
                     &   + EOS242*zt + EOS142 )*zsr   &
                     &   + ( EOS332*zt + EOS232 )*zt + EOS132 )*zsr   &
                     &   + ( ( EOS422*zt + EOS322 )*zt + EOS222 )*zt + EOS122 )*zsr   &
                     &   + ( ( ( EOS512*zt + EOS412 )*zt + EOS312 )*zt + EOS212 )*zt + EOS112
                  !
                  zn0  = ( ( ( ( ( EOS171*zsr   &
                     &   + EOS261*zt + EOS161 )*zsr   &
                     &   + ( EOS351*zt + EOS251 )*zt + EOS151 )*zsr   &
                     &   + ( ( EOS441*zt + EOS341 )*zt + EOS241 )*zt + EOS141 )*zsr   &
                     &   + ( ( ( EOS531*zt + EOS431 )*zt + EOS331 )*zt + EOS231 )*zt + EOS131 )*zsr   &
                     &   + ( ( ( ( EOS621*zt + EOS521 )*zt + EOS421 )*zt + EOS321 )*zt + EOS221 )*zt + EOS121 )*zsr   &
                     &   + ( ( ( ( ( EOS711*zt + EOS611 )*zt + EOS511 )*zt + EOS411 )*zt + EOS311 )*zt + EOS211 )*zt + EOS111
                  !
                  zn   = ( zn2 * zh + zn1 ) * zh + zn0
                  !
                  prhop(ji,jj,jk) = zn0 * ztm                           ! potential density referenced at the surface
                  !
                  prd(ji,jj,jk) = (  zn * r1_rau0 - 1._wp  ) * ztm      ! density anomaly (masked)
               END DO
            END DO
         END DO
         !
      CASE( 1 )                !==  simplified EOS  ==!
         !
         DO jk = 1, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zt  = pts  (ji,jj,jk,jp_tem) - 10._wp
                  zs  = pts  (ji,jj,jk,jp_sal) - 35._wp
                  zh  = pdep (ji,jj,jk)
                  ztm = tmask(ji,jj,jk)
                  !                                                     ! potential density referenced at the surface
                  zn =  - rn_a0 * ( 1._wp + 0.5_wp*rn_lambda1*zt ) * zt   &
                     &  + rn_b0 * ( 1._wp - 0.5_wp*rn_lambda2*zs ) * zs   &
                     &  - rn_nu * zt * zs
                  prhop(ji,jj,jk) = ( rau0 + zn ) * ztm
                  !                                                     ! density anomaly (masked)
                  zn = zn - ( rn_a0 * rn_mu1 * zt + rn_b0 * rn_mu2 * zs ) * zh
                  prd(ji,jj,jk) = zn * r1_rau0 * ztm
                  !
               END DO
            END DO
         END DO
         !
      END SELECT
      !
      IF(ln_ctl)   CALL prt_ctl( tab3d_1=prd, clinfo1=' eos-p: ', tab3d_2=prhop, clinfo2=' pot : ', ovlap=1, kdim=jpk )
      !
      IF( nn_timing == 1 )   CALL timing_stop('eos-p')
      !
   END SUBROUTINE eos_insitu_pot


   SUBROUTINE eos_insitu_2d( pts, pdep, prd )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE eos_insitu_2d  ***
      !!
      !! ** Purpose :   Compute the in situ density (ratio rho/rau0) from
      !!      potential temperature and salinity using an equation of state
      !!      defined through the namelist parameter nn_eos. * 2D field case
      !!
      !! ** Action  : - prd , the in situ density (no units) (unmasked)
      !!
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpts), INTENT(in   ) ::   pts   ! 1 : potential temperature  [Celcius]
      !                                                           ! 2 : salinity               [psu]
      REAL(wp), DIMENSION(jpi,jpj)     , INTENT(in   ) ::   pdep  ! depth                      [m]
      REAL(wp), DIMENSION(jpi,jpj)     , INTENT(  out) ::   prd   ! in situ density
      !
      INTEGER  ::   ji, jj, jk                ! dummy loop indices
      REAL(wp) ::   zt , zh , zs , zsr, ztm   ! local scalars
      REAL(wp) ::   zn , zn0, zn1, zn2        !   -      -
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )   CALL timing_start('eos2d')
      !
      prd(:,:) = 0._wp
      !
      SELECT CASE( nn_eos )
      !
      CASE( -1, 0 )                !==  polynomial TEOS-10 / EOS-80 ==!
         !
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               !
               zt    = pts  (ji,jj,jp_tem)            ! interpolated T
               zsr = SQRT( MAX( pts(ji,jj,jp_sal) , 0.1_wp ) )     ! square root salinity
               zh    = pdep (ji,jj)            ! depth at the partial step level
               !
               !
               zn2  = (   EOS133*zsr  + EOS223*zt + EOS123 )*zsr + ( EOS313*zt + EOS213 )*zt + EOS113
               !
               zn1  = ( ( ( EOS152*zsr   &
                  &   + EOS242*zt + EOS142 )*zsr   &
                  &   + ( EOS332*zt + EOS232 )*zt + EOS132 )*zsr   &
                  &   + ( ( EOS422*zt + EOS322 )*zt + EOS222 )*zt + EOS122 )*zsr   &
                  &   + ( ( ( EOS512*zt + EOS412 )*zt + EOS312 )*zt + EOS212 )*zt + EOS112
               !
               zn0  = ( ( ( ( ( EOS171*zsr   &
                  &   + EOS261*zt + EOS161 )*zsr   &
                  &   + ( EOS351*zt + EOS251 )*zt + EOS151 )*zsr   &
                  &   + ( ( EOS441*zt + EOS341 )*zt + EOS241 )*zt + EOS141 )*zsr   &
                  &   + ( ( ( EOS531*zt + EOS431 )*zt + EOS331 )*zt + EOS231 )*zt + EOS131 )*zsr   &
                  &   + ( ( ( ( EOS621*zt + EOS521 )*zt + EOS421 )*zt + EOS321 )*zt + EOS221 )*zt + EOS121 )*zsr   &
                  &   + ( ( ( ( ( EOS711*zt + EOS611 )*zt + EOS511 )*zt + EOS411 )*zt + EOS311 )*zt + EOS211 )*zt + EOS111
               !
               zn   = ( zn2 * zh + zn1 ) * zh + zn0
               !
               prd(ji,jj) = zn * r1_rau0 - 1._wp               ! unmasked in situ density anomaly
               !
            END DO
         END DO
         !
      CASE( 1 )                !==  simplified EOS  ==!
         !
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               !
               zt    = pts  (ji,jj,jp_tem)  - 10._wp
               zs    = pts  (ji,jj,jp_sal)  - 35._wp
               zh    = pdep (ji,jj)                         ! depth at the partial step level
               !
               zn =  - rn_a0 * ( 1._wp + 0.5_wp*rn_lambda1*zt + rn_mu1*zh ) * zt   &
                  &  + rn_b0 * ( 1._wp - 0.5_wp*rn_lambda2*zs - rn_mu2*zh ) * zs   &
                  &  - rn_nu * zt * zs
               !
               prd(ji,jj) = zn * r1_rau0               ! unmasked in situ density anomaly
               !
            END DO
         END DO
         !
      END SELECT
      !
      IF(ln_ctl)   CALL prt_ctl( tab2d_1=prd, clinfo1=' eos2d: ' )
      !
      IF( nn_timing == 1 )   CALL timing_stop('eos2d')
      !
   END SUBROUTINE eos_insitu_2d


   SUBROUTINE rab_3d( pts, pab )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE rab_3d  ***
      !!
      !! ** Purpose :   Calculates thermal/haline expansion ratio at T-points
      !!
      !! ** Method  :   calculates alpha / beta at T-points
      !!       * nn_eos = 0  : polynomial approximation of McDougall (1987)
      !!                       The alpha/beta is returned as 4-D array pab using the exact expression
      !!                       based on the JM95 equation of state.
      !!       * nn_eos = 1  : linear equation of state (temperature only)
      !!                       We return alpha and beta=0
      !!       * nn_eos = 2  : linear equation of state (temperature & salinity)
      !!                       We return alpha0 and beta0
      !!
      !! ** Action  : - pab     : thermal/haline expansion ratio at T-points
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(in   ) ::   pts   ! pot. temperature & salinity
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(  out) ::   pab   ! thermal/haline expansion ratio
      !
      INTEGER  ::   ji, jj, jk                ! dummy loop indices
      REAL(wp) ::   zt , zh , zs , zsr, ztm   ! local scalars
      REAL(wp) ::   zn , zn0, zn1, zn2        !   -      -
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )   CALL timing_start('rab_3d')
      !
      SELECT CASE ( nn_eos )
      !
      CASE( -1, 0 )                !==  polynomial TEOS-10 / EOS-80 ==!
         !
         DO jk = 1, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  !
                  zh  = fsdept(ji,jj,jk)                        ! depth
                  zt  = pts   (ji,jj,jk,jp_tem)                 ! conservative temperature
                  zsr = SQRT( MAX( pts(ji,jj,jk,jp_sal) , 0.1_wp ) )     ! square root salinity
                  ztm = tmask(ji,jj,jk)                         ! tmask
                  !
                  zn2 = ALP123*zsr + ALP213*zt + ALP113
                  !
                  zn1 = ( ( ALP142*zsr   &
                     &   + ALP232*zt + ALP132 )*zsr   &
                     &   + ( ALP322*zt + ALP222 )*zt + ALP122 )*zsr   &
                     &   + ( ( ALP412*zt + ALP312 )*zt + ALP212 )*zt + ALP112
                  !
                  zn0 = ( ( ( ( ALP161*zsr   &
                     &   + ALP251*zt + ALP151 )*zsr   &
                     &   + ( ALP341*zt + ALP241 )*zt + ALP141 )*zsr   &
                     &   + ( ( ALP431*zt + ALP331 )*zt + ALP231 )*zt + ALP131 )*zsr   &
                     &   + ( ( ( ALP521*zt + ALP421 )*zt + ALP321 )*zt + ALP221 )*zt + ALP121 )*zsr   &
                     &   + ( ( ( ( ALP611*zt + ALP511 )*zt + ALP411 )*zt + ALP311 )*zt + ALP211 )*zt + ALP111
                     !
                  zn   = ( zn2 * zh + zn1 ) * zh + zn0
                  !
                  pab(ji,jj,jk,jp_tem) = zn * r1_rau0 * ztm
                  !
                  !
                  zn2 = BET123*zsr + BET213*zt + BET113
                  !
                  zn1 = ( ( BET142*zsr   &
                     &   + BET232*zt + BET132 )*zsr   &
                     &   + ( BET322*zt + BET222 )*zt + BET122 )*zsr   &
                     &   + ( ( BET412*zt + BET312 )*zt + BET212 )*zt + BET112
                     !
                  zn0 = ( ( ( ( BET161*zsr   &
                     &   + BET251*zt + BET151 )*zsr   &
                     &   + ( BET341*zt + BET241 )*zt + BET141 )*zsr   &
                     &   + ( ( BET431*zt + BET331 )*zt + BET231 )*zt + BET131 )*zsr   &
                     &   + ( ( ( BET521*zt + BET421 )*zt + BET321 )*zt + BET221 )*zt + BET121 )*zsr   &
                     &   + ( ( ( ( BET611*zt + BET511 )*zt + BET411 )*zt + BET311 )*zt + BET211 )*zt + BET111
                     !
                  zn  = ( zn2 * zh + zn1 ) * zh + zn0
                  !
                  pab(ji,jj,jk,jp_sal) = zn / zsr * r1_rau0 * ztm
                  !
               END DO
            END DO
         END DO
         !
      CASE( 1 )                  !==  simplified EOS  ==!
         !
         DO jk = 1, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zt  = pts (ji,jj,jk,jp_tem) - 10._wp   ! pot. temperature anomaly (t-T0)
                  zs  = pts (ji,jj,jk,jp_sal) - 35._wp   ! abs. salinity anomaly (s-S0)
                  zh  = fsdept(ji,jj,jk)                 ! depth in meters at t-point
                  ztm = tmask(ji,jj,jk)                  ! land/sea bottom mask = surf. mask
                  !                                      ! alpha
                  pab(ji,jj,jk,jp_tem) = ( rn_a0 * ( 1._wp + rn_lambda1*zt + rn_mu1*zh ) + rn_nu*zs ) * r1_rau0 * ztm   ! alpha
                  pab(ji,jj,jk,jp_sal) = ( rn_b0 * ( 1._wp - rn_lambda2*zs - rn_mu2*zh ) - rn_nu*zt ) * r1_rau0 * ztm   ! beta                            ! beta
               END DO
            END DO
         END DO
         !
      CASE DEFAULT
         IF(lwp) WRITE(numout,cform_err)
         IF(lwp) WRITE(numout,*) '          bad flag value for nn_eos = ', nn_eos
         nstop = nstop + 1
         !
      END SELECT
      !
      IF( nn_timing == 1 )   CALL timing_stop('rab_3d')
      !
   END SUBROUTINE rab_3d


   SUBROUTINE rab_2d( pts, pdep, pab )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE rab_2d  ***
      !!
      !! ** Purpose :   Calculates thermal/haline expansion ratio for a 2d field (unmasked)
      !!
      !! ** Action  : - pab     : thermal/haline expansion ratio at T-points
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpts)    , INTENT(in   ) ::   pts    ! pot. temperature & salinity
      REAL(wp), DIMENSION(jpi,jpj)         , INTENT(in   ) ::   pdep   ! depth                  [m]
      REAL(wp), DIMENSION(jpi,jpj,jpts)    , INTENT(  out) ::   pab    ! thermal/haline expansion ratio
      !
      INTEGER  ::   ji, jj, jk                ! dummy loop indices
      REAL(wp) ::   zt , zh , zs , zsr, ztm   ! local scalars
      REAL(wp) ::   zn , zn0, zn1, zn2        !   -      -
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 ) CALL timing_start('rab_2d')
      !
      pab(:,:,:) = 0._wp
      !
      SELECT CASE ( nn_eos )
      !
      CASE( -1, 0 )                !==  polynomial TEOS-10 / EOS-80 ==!
         !
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               !
               zt    = pts  (ji,jj,jp_tem)            ! interpolated T
               zsr = SQRT( MAX( pts(ji,jj,jp_sal) , 0.1_wp ) )     ! square root salinity
               zh    = pdep (ji,jj)                   ! depth at the partial step level
               !
               !
               zn2 = ALP123*zsr + ALP213*zt + ALP113
               !
               zn1 = ( ( ALP142*zsr   &
                  &   + ALP232*zt + ALP132 )*zsr   &
                  &   + ( ALP322*zt + ALP222 )*zt + ALP122 )*zsr   &
                  &   + ( ( ALP412*zt + ALP312 )*zt + ALP212 )*zt + ALP112
               !
               zn0 = ( ( ( ( ALP161*zsr   &
                  &   + ALP251*zt + ALP151 )*zsr   &
                  &   + ( ALP341*zt + ALP241 )*zt + ALP141 )*zsr   &
                  &   + ( ( ALP431*zt + ALP331 )*zt + ALP231 )*zt + ALP131 )*zsr   &
                  &   + ( ( ( ALP521*zt + ALP421 )*zt + ALP321 )*zt + ALP221 )*zt + ALP121 )*zsr   &
                  &   + ( ( ( ( ALP611*zt + ALP511 )*zt + ALP411 )*zt + ALP311 )*zt + ALP211 )*zt + ALP111
               !
               zn   = ( zn2 * zh + zn1 ) * zh + zn0
               !
               pab(ji,jj,jp_tem) = zn * r1_rau0
               !
               !
               zn2 = BET123*zsr + BET213*zt + BET113
               !
               zn1 = ( ( BET142*zsr   &
                  &   + BET232*zt + BET132 )*zsr   &
                  &   + ( BET322*zt + BET222 )*zt + BET122 )*zsr   &
                  &   + ( ( BET412*zt + BET312 )*zt + BET212 )*zt + BET112
               !
               zn0 = ( ( ( ( BET161*zsr   &
                  &   + BET251*zt + BET151 )*zsr   &
                  &   + ( BET341*zt + BET241 )*zt + BET141 )*zsr   &
                  &   + ( ( BET431*zt + BET331 )*zt + BET231 )*zt + BET131 )*zsr   &
                  &   + ( ( ( BET521*zt + BET421 )*zt + BET321 )*zt + BET221 )*zt + BET121 )*zsr   &
                  &   + ( ( ( ( BET611*zt + BET511 )*zt + BET411 )*zt + BET311 )*zt + BET211 )*zt + BET111
               !
               zn   = ( zn2 * zh + zn1 ) * zh + zn0
               !
               pab(ji,jj,jp_sal) = zn / zsr * r1_rau0
               !
               !
            END DO
         END DO
         !
      CASE( 1 )                  !==  simplified EOS  ==!
         !
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               !
               zt    = pts  (ji,jj,jp_tem) - 10._wp   ! pot. temperature anomaly (t-T0)
               zs    = pts  (ji,jj,jp_sal) - 35._wp   ! abs. salinity anomaly (s-S0)
               zh    = pdep (ji,jj)                   ! depth at the partial step level
               !
               !                                    ! alpha
               pab(ji,jj,jp_tem) = ( rn_a0 * ( 1._wp + rn_lambda1*zt + rn_mu1*zh ) + rn_nu*zs ) * r1_rau0   ! alpha
               pab(ji,jj,jp_sal) = ( rn_b0 * ( 1._wp - rn_lambda2*zs - rn_mu2*zh ) - rn_nu*zt ) * r1_rau0   ! beta                            ! beta
               !
            END DO
         END DO
         !
      CASE DEFAULT
         IF(lwp) WRITE(numout,cform_err)
         IF(lwp) WRITE(numout,*) '          bad flag value for nn_eos = ', nn_eos
         nstop = nstop + 1
         !
      END SELECT
      !
      IF( nn_timing == 1 )   CALL timing_stop('rab_2d')
      !
   END SUBROUTINE rab_2d


   SUBROUTINE bn2( pts, pab, pn2 )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE bn2  ***
      !!
      !! ** Purpose :   Compute the local Brunt-Vaisala frequency at the 
      !!                time-step of the input arguments
      !!
      !! ** Method  :   pn2 = grav * (alpha dk[T] + beta dk[S] ) / e3w
      !!      where alpha and beta are given in pab, and computed on T-points.
      !!      N.B. N^2 is set one for all to zero at jk=1 in istate module.
      !!
      !! ** Action  :   pn2 : square of the brunt-vaisala frequency at w-point 
      !!
      !! References :   Vallis, Atmospheric and Oceanic Fluid Dynamics, 2006
      !!                Jackett and McDougall, J. Atmos. Ocean. Tech., 1995
      !!                McDougall, J. Phys. Oceano., 1987
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(in   ) ::  pts   ! pot. temperature and salinity   [Celcius,psu]
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(in   ) ::  pab   ! thermal/haline expansion coef.  [Celcius-1,psu-1]
      REAL(wp), DIMENSION(jpi,jpj,jpk     ), INTENT(  out) ::  pn2   ! Brunt-Vaisala frequency squared [1/s^2]
      !
      INTEGER  ::   ji, jj, jk      ! dummy loop indices
      REAL(wp) ::   zaw, zbw, zrw   ! local scalars
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 ) CALL timing_start('bn2')
      !
      DO jk = 2, jpkm1           ! interior points only (2=< jk =< jpkm1 )
         DO jj = 1, jpj          ! surface and bottom value set to zero one for all in istate.F90
            DO ji = 1, jpi
               zrw =   ( fsdepw(ji,jj,jk  ) - fsdept(ji,jj,jk) )   &
                  &  / ( fsdept(ji,jj,jk-1) - fsdept(ji,jj,jk) ) 
               !
               zaw = pab(ji,jj,jk,jp_tem) * (1. - zrw) + pab(ji,jj,jk-1,jp_tem) * zrw 
               zbw = pab(ji,jj,jk,jp_sal) * (1. - zrw) + pab(ji,jj,jk-1,jp_sal) * zrw
               !
               pn2(ji,jj,jk) = grav * (  zaw * ( pts(ji,jj,jk-1,jp_tem) - pts(ji,jj,jk,jp_tem) )     &
                  &                    - zbw * ( pts(ji,jj,jk-1,jp_sal) - pts(ji,jj,jk,jp_sal) )  )  &
                  &            / fse3w(ji,jj,jk) * tmask(ji,jj,jk)
            END DO
         END DO
      END DO
      !
      IF(ln_ctl)   CALL prt_ctl( tab3d_1=pn2, clinfo1=' bn2  : ', ovlap=1, kdim=jpk )
      !
      IF( nn_timing == 1 )   CALL timing_stop('bn2')
      !
   END SUBROUTINE bn2


   FUNCTION eos_pt_from_ct( ctmp, psal ) RESULT( ptmp )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE eos_pt_from_ct  ***
      !!
      !! ** Purpose :   Compute pot.temp. from cons. temp. [Celcius]
      !!
      !! ** Method  :   rational approximation (5/3th order) of TEOS-10 algorithm
      !!       checkvalue: pt=20.0239165517474 Celsius for s=35.7psu, t=20degC
      !!
      !! Reference  :   TEOS-10, UNESCO
      !!                Rational fit, see Roquet (2013)
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ) ::   ctmp   ! Cons. Temp [Celcius]
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ) ::   psal   ! salinity   [psu]
      ! Leave result array automatic rather than making explicitly allocated
      REAL(wp), DIMENSION(jpi,jpj) ::   ptmp   ! potential temperature [Celcius]
      !
      INTEGER  ::   ji, jj               ! dummy loop indices
      REAL(wp) ::   zt , zsr, ztm        ! local scalars
      REAL(wp) ::   zn , zd              ! local scalars
      !!----------------------------------------------------------------------
      !
      IF ( nn_timing == 1 )   CALL timing_start('eos_pt_from_ct')
      !
      DO jj = 1, jpj
          DO ji = 1, jpi
              zt  = ctmp   (ji,jj)
              zsr = SQRT( MAX( psal(ji,jj) , 0.1_wp ) )              ! square root salinity
              ztm = tmask(ji,jj,1)                          ! tmask
              !
              zn = ( ( ( ( -1.960202285944569e-04_wp*zsr    &
                  &    - 6.698743070819174e-05_wp*zt + 4.456449354084589e-03_wp )*zsr    &
                  &    + ( 1.424140773841990e-05_wp*zt - 1.961306826357777e-04_wp )*zt &
                  &        - 1.535458707717669e-02_wp )*zsr    &
                  &    + ( ( 2.814957335530553e-06_wp*zt + 1.534287192323960e-04_wp )*zt &
                  &        + 2.701363554476190e-02_wp )*zt - 1.869037589289360e-02_wp )*zsr    &
                  &    + ( ( ( -1.646758381788031e-08_wp*zt + 4.568111226988701e-07_wp )*zt &
                  &        - 7.853794523448087e-04_wp )*zt - 8.000001510387515e-03_wp )*zt &
                  &        - 1.132820958753811e-03_wp )*zsr    &
                  &    + ( ( ( ( -3.885399051933490e-09_wp*zt + 6.608853649947305e-07_wp )*zt &
                  &        - 1.507746029483144e-04_wp )*zt - 3.457378558412451e-03_wp )*zt &
                  &        - 7.616362942562558e-01_wp )*zt - 2.067472332529282e-01_wp
              !
              zd = ( ( 8.421053090759674e-04_wp*zsr    &
                  &    - 8.267668312138185e-04_wp*zt - 6.920238162292441e-02_wp )*zsr    &
                  &    + ( 5.063023899703211e-05_wp*zt + 1.488626985392304e-02_wp )*zt &
                  &        + 1.541411683853927e-01_wp )*zsr    &
                  &    + ( ( 6.537389208598346e-07_wp*zt + 1.856814792031252e-03_wp )*zt &
                  &        + 1.592477436776734e-01_wp )*zt + 1.406742653394891e+01_wp
              !
              ptmp(ji,jj) = ( zt + zn / zd ) * ztm
              !
          END DO
      END DO
      !
      IF( nn_timing == 1 )   CALL timing_stop('eos_pt_from_ct')
      !
   END FUNCTION eos_pt_from_ct


   FUNCTION eos_fzp( psal, pdep ) RESULT( ptf )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE eos_fzp  ***
      !!
      !! ** Purpose :   Compute the freezing point temperature [Celcius]
      !!
      !! ** Method  :   UNESCO freezing point (ptf) in Celcius is given by
      !!       ptf(t,z) = (-.0575+1.710523e-3*sqrt(abs(s))-2.154996e-4*s)*s - 7.53e-4*z
      !!       checkvalue: tf=-2.588567 Celsius for s=40psu, z=500m
      !!
      !! Reference  :   UNESCO tech. papers in the marine science no. 28. 1978
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   )           ::   psal   ! salinity   [psu]
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ), OPTIONAL ::   pdep   ! depth      [m]
      ! Leave result array automatic rather than making explicitly allocated
      REAL(wp), DIMENSION(jpi,jpj) ::   ptf   ! freezing temperature [Celcius]
      !
      INTEGER  ::   ji, jj               ! dummy loop indices
      REAL(wp) ::   zt , zsr             ! local scalars
      !!----------------------------------------------------------------------
      !
      SELECT CASE ( nn_eos )
      !
      CASE ( -1, 1 )                !==  CT,SA (TEOS-10 formulation) ==!
         !
         DO jj = 1, jpj
            DO ji = 1, jpi
               zsr= SQRT( ABS( psal(ji,jj) ) ) * 0.1_wp   ! square root salinity x 0.1
               ptf(ji,jj) = 0.017947064327968_wp &
                    &    + zsr*zsr*(-6.07609909992982_wp + zsr*(4.883198653547851_wp &
                    &    + zsr*(-11.8808160123054_wp + zsr*(13.34658511480257_wp &
                    &    + zsr*(-8.722761043208607_wp + 2.082038908808201_wp*zsr)))))
            END DO
         END DO
         !
         IF( PRESENT( pdep ) )   ptf(:,:) = ptf(:,:) - 7.69e-4 * pdep(:,:)
         !
      CASE ( 0 )                     !==  PT,SP (UNESCO formulation)  ==!
         !
         ptf(:,:) = ( - 0.0575_wp + 1.710523e-3_wp * SQRT( psal(:,:) )   &
            &                     - 2.154996e-4_wp *       psal(:,:)   ) * psal(:,:)
         !
         IF( PRESENT( pdep ) )   ptf(:,:) = ptf(:,:) - 7.53e-4 * pdep(:,:)
         !
      CASE DEFAULT
         IF(lwp) WRITE(numout,cform_err)
         IF(lwp) WRITE(numout,*) '          bad flag value for nn_eos = ', nn_eos
         nstop = nstop + 1
         !
      END SELECT
      !
   END FUNCTION eos_fzp


   SUBROUTINE eos_pen( pts, pab_pe, ppen )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE eos_pen  ***
      !!
      !! ** Purpose :   Calculates nonlinear anomalies of alpha_PE, beta_PE and PE at T-points
      !!
      !! ** Method  :   PE is defined analytically as the vertical 
      !!                   primitive of EOS times -g integrated between 0 and z>0.
      !!                pen is the nonlinear PE anomaly: ppen = ( PE - rau0 gz ) / rau0 gz - rd
      !!                                                      = 1/z * /int_0^z rd dz - rd 
      !!                                where rd is the density anomaly (see eos_rhd function)
      !!                ab_pe are partial derivatives of PE anomaly with respect to T and S:
      !!                    ab_pe(1) = - 1/(rau0 gz) * dPE/dT + drd/dT = - d(pen)/dT
      !!                    ab_pe(2) =   1/(rau0 gz) * dPE/dS + drd/dS =   d(pen)/dS
      !!
      !!       * nn_eos = -1 : polynomial TEOS-10
      !!       * nn_eos =  0 : Jackett and McDougall (1995) (should not be used, using polynomial TEOS-10 formulation)
      !!       * nn_eos =  1 : Vallis equation of state (Vallis 2006)
      !!
      !! ** Action  : - pen         : PE anomaly given at T-points
      !!            : - pab_pe  : given at T-points
      !!                    pab_pe(:,:,:,jp_tem) is alpha_pe
      !!                    pab_pe(:,:,:,jp_sal) is beta_pe
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(in   ) ::   pts     ! pot. temperature & salinity
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(  out) ::   pab_pe  ! alpha_pe and beta_pe
      REAL(wp), DIMENSION(jpi,jpj,jpk)     , INTENT(  out) ::   ppen     ! potential energy anomaly
      !
      INTEGER  ::   ji, jj, jk               ! dummy loop indices
      REAL(wp) ::   zt , zh , zsr, ztm , zs  ! local scalars
      REAL(wp) ::   zn
      !!----------------------------------------------------------------------
      !
      IF ( nn_timing == 1 )   CALL timing_start('eos_pen')
      !
      SELECT CASE ( nn_eos )
      !
      CASE ( -1 , 0 )               ! polyTEOS10 (used also for JM95 case as an approximation)
         !
         DO jk = 1, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zt  = pts (ji,jj,jk,jp_tem)
                  zh  = fsdept(ji,jj,jk)                              ! depth
                  zsr = SQRT( MAX( pts(ji,jj,jk,jp_sal) , 0.1_wp ) )  ! square root salinity
                  ztm = tmask(ji,jj,jk)                               ! tmask
                  !
                  !
                  zn  = ( ( ( PEN133*zsr   &
                     &   + PEN223*zt + PEN123 )*zsr   &
                     &   + ( PEN313*zt + PEN213 )*zt + PEN113 )*zh   &
                     &   + ( ( ( PEN152*zsr   &
                     &   + PEN242*zt + PEN142 )*zsr   &
                     &   + ( PEN332*zt + PEN232 )*zt + PEN132 )*zsr   &
                     &   + ( ( PEN422*zt + PEN322 )*zt + PEN222 )*zt + PEN122 )*zsr   &
                     &   + ( ( ( PEN512*zt + PEN412 )*zt + PEN312 )*zt + PEN212 )*zt + PEN112 )*zh
                  !
                  !                              ! potential energy non-linear anomaly
                  ppen(ji,jj,jk)  = zn * r1_rau0 * ztm
                  !
                  !
                  zn  = ( ( ( APE123 )*zsr   &
                     &   + APE213*zt + APE113 )*zh   &
                     &   + ( ( ( APE142 )*zsr   &
                     &   + APE232*zt + APE132 )*zsr   &
                     &   + ( APE322*zt + APE222 )*zt + APE122 )*zsr   &
                     &   + ( ( APE412*zt + APE312 )*zt + APE212 )*zt + APE112 )*zh
                  !
                  !                              ! alphaPE non-linear anomaly
                  pab_pe(ji,jj,jk,jp_tem) = zn * r1_rau0 * ztm
                  !
                  !
                  zn  = ( ( ( BPE123 )*zsr   &
                     &   + BPE213*zt + BPE113 )*zh   &
                     &   + ( ( ( BPE142 )*zsr   &
                     &   + BPE232*zt + BPE132 )*zsr   &
                     &   + ( BPE322*zt + BPE222 )*zt + BPE122 )*zsr   &
                     &   + ( ( BPE412*zt + BPE312 )*zt + BPE212 )*zt + BPE112 )*zh
                  !
                  !                              ! betaPE non-linear anomaly
                  pab_pe(ji,jj,jk,jp_sal) = zn / zsr * r1_rau0 * ztm
                  !
               END DO
            END DO
         END DO
         !
      CASE( 1 )                !==  Vallis (2006) simplified EOS  ==!
         !
         DO jk = 1, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zt  = pts(ji,jj,jk,jp_tem) - 10._wp  ! temperature anomaly (t-T0)
                  zs = pts (ji,jj,jk,jp_sal) - 35._wp  ! abs. salinity anomaly (s-S0)
                  zh  = fsdept(ji,jj,jk)               ! depth in meters  at t-point
                  ztm = tmask(ji,jj,jk)                ! tmask
                  zn  = 0.5_wp * zh * r1_rau0 * ztm
                  !                                    ! Potential Energy
                  ppen(ji,jj,jk) = ( rn_a0 * rn_mu1 * zt + rn_b0 * rn_mu2 * zs ) * zn
                  !                                    ! alphaPE
                  pab_pe(ji,jj,jk,jp_tem) = - rn_a0 * rn_mu1 * zn
                  pab_pe(ji,jj,jk,jp_sal) =   rn_b0 * rn_mu2 * zn
                  !
               END DO
            END DO
         END DO
         !
      CASE DEFAULT
         IF(lwp) WRITE(numout,cform_err)
         IF(lwp) WRITE(numout,*) '          bad flag value for nn_eos = ', nn_eos
         nstop = nstop + 1
         !
      END SELECT
      !
      IF( nn_timing == 1 )   CALL timing_stop('eos_pen')
      !
   END SUBROUTINE eos_pen


   SUBROUTINE eos_init
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE eos_init  ***
      !!
      !! ** Purpose :   initializations for the equation of state
      !!
      !! ** Method  :   Read the namelist nameos and control the parameters
      !!----------------------------------------------------------------------
      INTEGER  ::   ios   ! local integer
      !!
      NAMELIST/nameos/ nn_eos, ln_useCT, rn_a0, rn_b0, rn_lambda1, rn_mu1,   &
         &                                             rn_lambda2, rn_mu2, rn_nu
      !!----------------------------------------------------------------------
      !
      REWIND( numnam_ref )              ! Namelist nameos in reference namelist : equation of state
      READ  ( numnam_ref, nameos, IOSTAT = ios, ERR = 901 )
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'nameos in reference namelist', lwp )
      !
      REWIND( numnam_cfg )              ! Namelist nameos in configuration namelist : equation of state
      READ  ( numnam_cfg, nameos, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'nameos in configuration namelist', lwp )
      WRITE( numond, nameos )
      !
      rau0        = 1026._wp      !: volumic mass of reference     [kg/m3]
      rcp         = 3992._wp      !: heat capacity                 [J/K]
      !
      IF(lwp) THEN                ! Control print
         WRITE(numout,*)
         WRITE(numout,*) 'eos_init : equation of state'
         WRITE(numout,*) '~~~~~~~~'
         WRITE(numout,*) '          Namelist nameos : set eos parameters'
         WRITE(numout,*) '             flag for eq. of state and N^2  nn_eos   = ', nn_eos
         IF( ln_useCT )   THEN
            WRITE(numout,*) '             model uses Conservative Temperature'
            WRITE(numout,*) '             Important: model must be initialized with CT and SA fields'
         ENDIF
      ENDIF
      !
      SELECT CASE( nn_eos )         ! check option
      !
      CASE( -1 )                       !==  polynomial TEOS-10  ==!
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) '          use of TEOS-10 equation of state (cons. temp. and abs. salinity)'
         !
         EOS111 =  9.9984112007e+02_wp
         EOS211 =  6.4904564681e-02_wp
         EOS311 = -8.1866708294e-03_wp
         EOS411 =  8.4327846386e-05_wp
         EOS511 = -9.9853850905e-07_wp
         EOS611 =  9.5342506671e-09_wp
         EOS711 = -5.5942725501e-11_wp
         EOS121 =  4.8745609554e-04_wp
         EOS221 = -7.9137653036e-04_wp
         EOS321 =  1.1241300058e-04_wp
         EOS421 = -4.2658318975e-06_wp
         EOS521 =  2.7589924936e-08_wp
         EOS621 =  5.6573234128e-10_wp
         EOS131 =  8.2409990767e-01_wp
         EOS231 = -3.6746935996e-03_wp
         EOS331 =  2.8088034997e-05_wp
         EOS431 =  3.9432934562e-07_wp
         EOS531 = -8.6081969467e-09_wp
         EOS141 = -8.5860758561e-03_wp
         EOS241 =  1.0048170774e-04_wp
         EOS341 = -1.1219591564e-06_wp
         EOS441 = -1.4550322467e-09_wp
         EOS151 =  1.2633903654e-03_wp
         EOS251 = -7.2707200365e-06_wp
         EOS351 =  2.6241351559e-08_wp
         EOS161 = -7.4242043203e-05_wp
         EOS261 =  3.8374457797e-07_wp
         EOS171 =  1.2722088050e-06_wp
         EOS112 =  4.3845092294e-04_wp
         EOS212 = -3.5237528758e-05_wp
         EOS312 =  5.7586626030e-07_wp
         EOS412 = -6.3139238271e-09_wp
         EOS512 =  3.9434377793e-11_wp
         EOS122 = -7.6672754907e-06_wp
         EOS222 =  9.0194877799e-08_wp
         EOS322 = -1.6940551903e-09_wp
         EOS422 =  3.1783762285e-11_wp
         EOS132 = -5.9870305749e-06_wp
         EOS232 =  1.0325176608e-07_wp
         EOS332 = -8.0185331501e-10_wp
         EOS142 = -5.6324524012e-07_wp
         EOS242 =  6.6221210305e-10_wp
         EOS152 =  3.9558159454e-08_wp
         EOS113 = -1.6847693220e-09_wp
         EOS213 =  5.3653075114e-10_wp
         EOS313 = -1.0864160933e-11_wp
         EOS123 = -1.6883150438e-09_wp
         EOS223 =  6.5982407189e-12_wp
         EOS133 =  2.7370939782e-10_wp
         !
         ALP111 = -6.4904564681e-02_wp
         ALP211 =  1.6373341659e-02_wp
         ALP311 = -2.5298353916e-04_wp
         ALP411 =  3.9941540362e-06_wp
         ALP511 = -4.7671253336e-08_wp
         ALP611 =  3.3565635301e-10_wp
         ALP121 =  7.9137653036e-04_wp
         ALP221 = -2.2482600117e-04_wp
         ALP321 =  1.2797495693e-05_wp
         ALP421 = -1.1035969974e-07_wp
         ALP521 = -2.8286617064e-09_wp
         ALP131 =  3.6746935996e-03_wp
         ALP231 = -5.6176069994e-05_wp
         ALP331 = -1.1829880369e-06_wp
         ALP431 =  3.4432787787e-08_wp
         ALP141 = -1.0048170774e-04_wp
         ALP241 =  2.2439183129e-06_wp
         ALP341 =  4.3650967401e-09_wp
         ALP151 =  7.2707200365e-06_wp
         ALP251 = -5.2482703118e-08_wp
         ALP161 = -3.8374457797e-07_wp
         ALP112 =  3.5237528758e-05_wp
         ALP212 = -1.1517325206e-06_wp
         ALP312 =  1.8941771481e-08_wp
         ALP412 = -1.5773751117e-10_wp
         ALP122 = -9.0194877799e-08_wp
         ALP222 =  3.3881103805e-09_wp
         ALP322 = -9.5351286854e-11_wp
         ALP132 = -1.0325176608e-07_wp
         ALP232 =  1.6037066300e-09_wp
         ALP142 = -6.6221210305e-10_wp
         ALP113 = -5.3653075114e-10_wp
         ALP213 =  2.1728321866e-11_wp
         ALP123 = -6.5982407189e-12_wp
         !
         BET111 =  2.4372804777e-04_wp
         BET211 = -3.9568826518e-04_wp
         BET311 =  5.6206500292e-05_wp
         BET411 = -2.1329159488e-06_wp
         BET511 =  1.3794962468e-08_wp
         BET611 =  2.8286617064e-10_wp
         BET121 =  8.2409990767e-01_wp
         BET221 = -3.6746935996e-03_wp
         BET321 =  2.8088034997e-05_wp
         BET421 =  3.9432934562e-07_wp
         BET521 = -8.6081969467e-09_wp
         BET131 = -1.2879113784e-02_wp
         BET231 =  1.5072256162e-04_wp
         BET331 = -1.6829387347e-06_wp
         BET431 = -2.1825483700e-09_wp
         BET141 =  2.5267807308e-03_wp
         BET241 = -1.4541440073e-05_wp
         BET341 =  5.2482703118e-08_wp
         BET151 = -1.8560510801e-04_wp
         BET251 =  9.5936144494e-07_wp
         BET161 =  3.8166264151e-06_wp
         BET112 = -3.8336377454e-06_wp
         BET212 =  4.5097438900e-08_wp
         BET312 = -8.4702759513e-10_wp
         BET412 =  1.5891881142e-11_wp
         BET122 = -5.9870305749e-06_wp
         BET222 =  1.0325176608e-07_wp
         BET322 = -8.0185331501e-10_wp
         BET132 = -8.4486786018e-07_wp
         BET232 =  9.9331815458e-10_wp
         BET142 =  7.9116318907e-08_wp
         BET113 = -8.4415752190e-10_wp
         BET213 =  3.2991203594e-12_wp
         BET123 =  2.7370939782e-10_wp
         !
         PEN112 = -2.1922546147e-04_wp
         PEN212 =  1.7618764379e-05_wp
         PEN312 = -2.8793313015e-07_wp
         PEN412 =  3.1569619136e-09_wp
         PEN512 = -1.9717188896e-11_wp
         PEN122 =  3.8336377454e-06_wp
         PEN222 = -4.5097438900e-08_wp
         PEN322 =  8.4702759513e-10_wp
         PEN422 = -1.5891881142e-11_wp
         PEN132 =  2.9935152875e-06_wp
         PEN232 = -5.1625883042e-08_wp
         PEN332 =  4.0092665751e-10_wp
         PEN142 =  2.8162262006e-07_wp
         PEN242 = -3.3110605153e-10_wp
         PEN152 = -1.9779079727e-08_wp
         PEN113 =  1.1231795480e-09_wp
         PEN213 = -3.5768716743e-10_wp
         PEN313 =  7.2427739554e-12_wp
         PEN123 =  1.1255433625e-09_wp
         PEN223 = -4.3988271459e-12_wp
         PEN133 = -1.8247293188e-10_wp
         !
         APE112 = -1.7618764379e-05_wp
         APE212 =  5.7586626030e-07_wp
         APE312 = -9.4708857407e-09_wp
         APE412 =  7.8868755586e-11_wp
         APE122 =  4.5097438900e-08_wp
         APE222 = -1.6940551903e-09_wp
         APE322 =  4.7675643427e-11_wp
         APE132 =  5.1625883042e-08_wp
         APE232 = -8.0185331501e-10_wp
         APE142 =  3.3110605153e-10_wp
         APE113 =  3.5768716743e-10_wp
         APE213 = -1.4485547911e-11_wp
         APE123 =  4.3988271459e-12_wp
         !
         BPE112 =  1.9168188727e-06_wp
         BPE212 = -2.2548719450e-08_wp
         BPE312 =  4.2351379756e-10_wp
         BPE412 = -7.9459405711e-12_wp
         BPE122 =  2.9935152875e-06_wp
         BPE222 = -5.1625883042e-08_wp
         BPE322 =  4.0092665751e-10_wp
         BPE132 =  4.2243393009e-07_wp
         BPE232 = -4.9665907729e-10_wp
         BPE142 = -3.9558159454e-08_wp
         BPE113 =  5.6277168127e-10_wp
         BPE213 = -2.1994135730e-12_wp
         BPE123 = -1.8247293188e-10_wp
         !
      CASE( 0 )                        !==  polynomial EOS-80 formulation  ==!
         !
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) '          use of EOS-80 equation of state (pot. temp. and pract. salinity)'
         !
         EOS111 =  9.9984240021e+02_wp
         EOS211 =  6.8051135248e-02_wp
         EOS311 = -9.0991456041e-03_wp
         EOS411 =  9.9238754931e-05_wp
         EOS511 = -1.0602210389e-06_wp
         EOS611 =  5.1296071368e-09_wp
         EOS711 =  1.1887242136e-11_wp
         EOS121 = -1.9799119661e-04_wp
         EOS221 = -1.6676932531e-05_wp
         EOS321 =  3.2084304200e-06_wp
         EOS421 = -4.8983568948e-08_wp
         EOS521 = -1.5610111448e-09_wp
         EOS621 =  2.8366680541e-11_wp
         EOS131 =  8.2448582544e-01_wp
         EOS231 = -4.0764319966e-03_wp
         EOS331 =  7.4898579574e-05_wp
         EOS431 = -7.7884546946e-07_wp
         EOS531 =  5.0405569756e-09_wp
         EOS141 = -5.7150204345e-03_wp
         EOS241 =  1.0157923923e-04_wp
         EOS341 = -1.6325795501e-06_wp
         EOS441 = -1.6144820180e-09_wp
         EOS151 =  4.8005813592e-04_wp
         EOS251 =  1.6199868493e-07_wp
         EOS351 =  7.8805435082e-09_wp
         EOS161 =  1.9978872493e-07_wp
         EOS261 = -3.1437087783e-08_wp
         EOS171 =  2.5463113215e-08_wp
         EOS112 =  4.3756680685e-04_wp
         EOS212 = -3.7298355935e-05_wp
         EOS312 =  6.5348660941e-07_wp
         EOS412 = -7.2558835910e-09_wp
         EOS512 =  3.8273323221e-11_wp
         EOS122 =  1.4369434305e-06_wp
         EOS222 =  5.6521871685e-08_wp
         EOS322 = -1.2136936896e-08_wp
         EOS422 =  2.1030996727e-10_wp
         EOS132 = -1.0262750144e-05_wp
         EOS232 =  2.0858571138e-07_wp
         EOS332 = -1.4826357415e-09_wp
         EOS142 =  8.5860757162e-08_wp
         EOS242 = -5.2140945761e-09_wp
         EOS152 =  8.2162682979e-09_wp
         EOS113 = -5.5011801985e-09_wp
         EOS213 =  5.7020541017e-10_wp
         EOS313 = -1.0193603551e-11_wp
         EOS123 = -1.0893703284e-10_wp
         EOS223 = -4.1944954508e-12_wp
         EOS133 =  1.1857534169e-10_wp
         !
         ALP111 = -6.8051135248e-02_wp
         ALP211 =  1.8198291208e-02_wp
         ALP311 = -2.9771626479e-04_wp
         ALP411 =  4.2408841556e-06_wp
         ALP511 = -2.5648035684e-08_wp
         ALP611 = -7.1323452819e-11_wp
         ALP121 =  1.6676932531e-05_wp
         ALP221 = -6.4168608399e-06_wp
         ALP321 =  1.4695070684e-07_wp
         ALP421 =  6.2440445791e-09_wp
         ALP521 = -1.4183340270e-10_wp
         ALP131 =  4.0764319966e-03_wp
         ALP231 = -1.4979715915e-04_wp
         ALP331 =  2.3365364084e-06_wp
         ALP431 = -2.0162227903e-08_wp
         ALP141 = -1.0157923923e-04_wp
         ALP241 =  3.2651591003e-06_wp
         ALP341 =  4.8434460540e-09_wp
         ALP151 = -1.6199868493e-07_wp
         ALP251 = -1.5761087016e-08_wp
         ALP161 =  3.1437087783e-08_wp
         ALP112 =  3.7298355935e-05_wp
         ALP212 = -1.3069732188e-06_wp
         ALP312 =  2.1767650773e-08_wp
         ALP412 = -1.5309329288e-10_wp
         ALP122 = -5.6521871685e-08_wp
         ALP222 =  2.4273873792e-08_wp
         ALP322 = -6.3092990180e-10_wp
         ALP132 = -2.0858571138e-07_wp
         ALP232 =  2.9652714830e-09_wp
         ALP142 =  5.2140945761e-09_wp
         ALP113 = -5.7020541017e-10_wp
         ALP213 =  2.0387207103e-11_wp
         ALP123 =  4.1944954508e-12_wp
         !
         BET111 = -9.8995598303e-05_wp
         BET211 = -8.3384662657e-06_wp
         BET311 =  1.6042152100e-06_wp
         BET411 = -2.4491784474e-08_wp
         BET511 = -7.8050557238e-10_wp
         BET611 =  1.4183340270e-11_wp
         BET121 =  8.2448582544e-01_wp
         BET221 = -4.0764319966e-03_wp
         BET321 =  7.4898579574e-05_wp
         BET421 = -7.7884546946e-07_wp
         BET521 =  5.0405569756e-09_wp
         BET131 = -8.5725306517e-03_wp
         BET231 =  1.5236885884e-04_wp
         BET331 = -2.4488693252e-06_wp
         BET431 = -2.4217230270e-09_wp
         BET141 =  9.6011627184e-04_wp
         BET241 =  3.2399736986e-07_wp
         BET341 =  1.5761087016e-08_wp
         BET151 =  4.9947181231e-07_wp
         BET251 = -7.8592719459e-08_wp
         BET161 =  7.6389339644e-08_wp
         BET112 =  7.1847171523e-07_wp
         BET212 =  2.8260935842e-08_wp
         BET312 = -6.0684684481e-09_wp
         BET412 =  1.0515498363e-10_wp
         BET122 = -1.0262750144e-05_wp
         BET222 =  2.0858571138e-07_wp
         BET322 = -1.4826357415e-09_wp
         BET132 =  1.2879113574e-07_wp
         BET232 = -7.8211418642e-09_wp
         BET142 =  1.6432536596e-08_wp
         BET113 = -5.4468516419e-11_wp
         BET213 = -2.0972477254e-12_wp
         BET123 =  1.1857534169e-10_wp
         !
         PEN112 = -2.1878340343e-04_wp
         PEN212 =  1.8649177967e-05_wp
         PEN312 = -3.2674330470e-07_wp
         PEN412 =  3.6279417955e-09_wp
         PEN512 = -1.9136661610e-11_wp
         PEN122 = -7.1847171523e-07_wp
         PEN222 = -2.8260935842e-08_wp
         PEN322 =  6.0684684481e-09_wp
         PEN422 = -1.0515498363e-10_wp
         PEN132 =  5.1313750720e-06_wp
         PEN232 = -1.0429285569e-07_wp
         PEN332 =  7.4131787075e-10_wp
         PEN142 = -4.2930378581e-08_wp
         PEN242 =  2.6070472881e-09_wp
         PEN152 = -4.1081341490e-09_wp
         PEN113 =  3.6674534657e-09_wp
         PEN213 = -3.8013694011e-10_wp
         PEN313 =  6.7957357010e-12_wp
         PEN123 =  7.2624688558e-11_wp
         PEN223 =  2.7963303006e-12_wp
         PEN133 = -7.9050227794e-11_wp
         !
         APE112 = -1.8649177967e-05_wp
         APE212 =  6.5348660941e-07_wp
         APE312 = -1.0883825386e-08_wp
         APE412 =  7.6546646441e-11_wp
         APE122 =  2.8260935842e-08_wp
         APE222 = -1.2136936896e-08_wp
         APE322 =  3.1546495090e-10_wp
         APE132 =  1.0429285569e-07_wp
         APE232 = -1.4826357415e-09_wp
         APE142 = -2.6070472881e-09_wp
         APE113 =  3.8013694011e-10_wp
         APE213 = -1.3591471402e-11_wp
         APE123 = -2.7963303006e-12_wp
         !
         BPE112 = -3.5923585761e-07_wp
         BPE212 = -1.4130467921e-08_wp
         BPE312 =  3.0342342240e-09_wp
         BPE412 = -5.2577491816e-11_wp
         BPE122 =  5.1313750720e-06_wp
         BPE222 = -1.0429285569e-07_wp
         BPE322 =  7.4131787075e-10_wp
         BPE132 = -6.4395567871e-08_wp
         BPE232 =  3.9105709321e-09_wp
         BPE142 = -8.2162682979e-09_wp
         BPE113 =  3.6312344279e-11_wp
         BPE213 =  1.3981651503e-12_wp
         BPE123 = -7.9050227794e-11_wp
         !
      CASE( 1 )                        !==  Simplified EOS     ==!
         IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) '          use of simplified eos:    rhd(dT=T-10,dS=S-35,Z) = '
            WRITE(numout,*) '             [-a0*(1+lambda1/2*dT+mu1*Z)*dT + b0*(1+lambda2/2*dT+mu2*Z)*dS - nu*dT*dS]/rau0'
            WRITE(numout,*)
            WRITE(numout,*) '             thermal exp. coef.    rn_a0      = ', rn_a0
            WRITE(numout,*) '             saline  cont. coef.   rn_b0      = ', rn_b0
            WRITE(numout,*) '             cabbeling coef.       rn_lambda1 = ', rn_lambda1
            WRITE(numout,*) '             cabbeling coef.       rn_lambda2 = ', rn_lambda2
            WRITE(numout,*) '             thermobar. coef.      rn_mu1     = ', rn_mu1
            WRITE(numout,*) '             thermobar. coef.      rn_mu2     = ', rn_mu2
            WRITE(numout,*) '             2nd cabbel. coef.     rn_nu      = ', rn_nu
            WRITE(numout,*) '               Caution: rn_beta0=0 incompatible with ddm parameterization '
         ENDIF
         !
      CASE DEFAULT                     !==  ERROR in nn_eos  ==!
         WRITE(ctmp1,*) '          bad flag value for nn_eos = ', nn_eos
         CALL ctl_stop( ctmp1 )
         !
      END SELECT
      !
      r1_rau0     = 1._wp / rau0
      r1_rcp      = 1._wp / rcp
      r1_rau0_rcp = 1._wp / ( rau0 * rcp )
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) '          volumic mass of reference           rau0  = ', rau0   , ' kg/m^3'
      IF(lwp) WRITE(numout,*) '          1. / rau0                        r1_rau0  = ', r1_rau0, ' m^3/kg'
      IF(lwp) WRITE(numout,*) '          ocean specific heat                 rcp   = ', rcp    , ' J/Kelvin'
      IF(lwp) WRITE(numout,*) '          1. / ( rau0 * rcp )           r1_rau0_rcp = ', r1_rau0_rcp
      !
   END SUBROUTINE eos_init

   !!======================================================================
END MODULE eosbn2
